==> nfs-tests-0.3.tar.gz: nfs-tests-0.3/okir/bin/interrupt.c <==
/*
 * Run one or more tests and interrupt by sending it a signal
 *
 * Copyright (C) 1998, Olaf Kirch <okir@monad.swb.de>
 */
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <ctype.h>
#include <sys/wait.h>

static int		getsig(const char *);

static void
usage(int code)
{
	printf("usage:\n"
		"interrupt [-s sig] [-t delay] [-m message] [-n] "
		"-- command [ -- command ]\n");
	exit(code);
}

static void
catch(int sig)
{
	return;
}

static int	childdied = 0;
static int	terminate = 0;

static void
reaper(int sig)
{
	childdied++;
}

static void
terminator(int sig)
{
	terminate = sig;
}

int
main(int argc, char **argv)
{
	struct sigaction sac;
	char	*message = "Test", **cargv;
	int	signo = SIGINT;
	int	delay = 2;
	int	mayexit = 0;
	int	c, status, i, nr, child[16];

	while ((c = getopt(argc, argv, "m:ns:t:h")) != -1) {
		switch (c) {
		case 'm':
			message = optarg;
			break;
		case 'n':
			mayexit = 1;
			break;
		case 's':
			signo = getsig(optarg);
			break;
		case 't':
			delay = atoi(optarg);
			break;
		case 'h':
			usage(0);
		case '?':
			usage(1);
		}
	}

	if (optind == argc) {
		printf("Missing command\n");
		usage(1);
	}

	memset(&sac, 0, sizeof(sac));
	sigemptyset(&sac.sa_mask);

	sac.sa_handler = reaper;
	sigaction(SIGCHLD, &sac, NULL);
	sac.sa_handler = terminator;
	sigaction(SIGHUP,  &sac, NULL);
	sigaction(SIGTERM, &sac, NULL);
	sigaction(SIGINT,  &sac, NULL);
	sigaction(SIGQUIT, &sac, NULL);
	sigaction(SIGPIPE, &sac, NULL);

	printf("%s starting (duration <= %d seconds).\n", message, delay);
	nr = 0;
	do {
		cargv = argv + optind;
		while (optind < argc && strcmp(argv[optind], "--"))
			optind++;
		if (optind < argc)
			argv[optind++] = NULL;
		if ((c = fork()) < 0) {
			perror("fork");
			exit(1);
		}
		if (c == 0) {
			sac.sa_handler = catch;
			sigaction(SIGUSR1, &sac, NULL);
			pause();
			execvp(cargv[0], cargv);
			perror(cargv[0]);
			exit(1);
		}
		child[nr++] = c;
	} while (optind < argc);

	sleep(2);
	for (i = 0; i < nr; i++)
		kill(child[i], SIGUSR1);

	while (delay && childdied < nr) {
		delay = sleep(delay);
		if (terminate) {
			printf("%s terminated.\n", message);
			for (i = 0; i < nr; i++)
				kill(child[i], terminate);
			exit(1);
		}
	}

	for (i = 0; i < nr; i++)
		kill(child[i], signo);

	/* Wait up to 5 seconds for children to finish */
	delay = 5;
	while (delay && childdied < nr)
		delay = sleep(delay);

	if (childdied < nr) {
		printf("Children still running -- using brute force.\n");
		for (i = 0; i < nr; i++)
			kill(child[i], SIGKILL);
	}

	for (i = 0; i < nr; i++) {
		if ((c = wait(&status)) < 0) {
			perror("wait");
			exit(1);
		}

		if (WIFEXITED(status)) {
			const char	*blurb;

			if (mayexit) {
				if (WEXITSTATUS(status) == 0)
					continue;
				blurb = "with non-zero";
			} else {
				blurb = "prior to signal;";
			}
			printf("Child %d terminated %s exit status %d\n",
				c, blurb, WEXITSTATUS(status));
			goto fail;
		} else
		if (WIFSIGNALED(status) && WTERMSIG(status) != signo) {
			printf("Child %d died from signal %d (sent %d)\n",
				c, WTERMSIG(status), signo);
			goto fail;
		}
	}

	printf("%s passed.\n", message);
	return 0;

fail:
	printf("%s failed.\n", message);
	return 1;
}

static int
getsig(const char *spec)
{
	if (isdigit(spec[0]))
		return atoi(spec);
	if (!strcasecmp(spec, "INT"))
		return SIGINT;
	if (!strcasecmp(spec, "TERM"))
		return SIGTERM;
	if (!strcasecmp(spec, "KILL"))
		return SIGKILL;
	if (!strcasecmp(spec, "HUP"))
		return SIGHUP;
	if (!strcasecmp(spec, "ALRM"))
		return SIGALRM;

	printf("unexpected signal name %s\n", spec);
	exit(1);
}

==> nfs-tests-0.5.tar.gz: nfs-tests-0.5/okir/bin/interrupt.c <==
/*
 * Run one or more tests and interrupt by sending it a signal
 *
 * Copyright (C) 1998, Olaf Kirch <okir@monad.swb.de>
 *
 * -s sig	signal to send i norder to terminate children
 * -t delay	time after which children should be signalled
 * -m msg	Message to be printed
 * -n		If set, children may exit. If not set, consider
 *		it an error when a child terminates for reasons
 *		other than having received the specified signal.
 */
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <ctype.h>
#include <sys/wait.h>

static int		getsig(const char *);

static void
usage(int code)
{
	printf("usage:\n"
		"interrupt [-s sig] [-t delay] [-m message] [-n] "
		"-- command [ -- command ...]\n");
	exit(code);
}

static void
catch(int sig)
{
	return;
}

static int	childdied = 0;
static int	terminate = 0;

static void
reaper(int sig)
{
	childdied++;
}

static void
terminator(int sig)
{
	terminate = sig;
}

int
main(int argc, char **argv)
{
	struct sigaction sac;
	char	*message = "Test", **cargv;
	int	signo = SIGINT;
	int	delay = 2;
	int	mayexit = 0;
	int	c, status, i, nr, child[16];

	while ((c = getopt(argc, argv, "m:ns:t:h")) != -1) {
		switch (c) {
		case 'm':
			message = optarg;
			break;
		case 'n':
			mayexit = 1;
			break;
		case 's':
			signo = getsig(optarg);
			break;
		case 't':
			delay = atoi(optarg);
			break;
		case 'h':
			usage(0);
		case '?':
			usage(1);
		}
	}

	if (optind == argc) {
		printf("Missing command\n");
		usage(1);
	}

	memset(&sac, 0, sizeof(sac));
	sigemptyset(&sac.sa_mask);

	sac.sa_handler = reaper;
	sigaction(SIGCHLD, &sac, NULL);
	sac.sa_handler = terminator;
	sigaction(SIGHUP,  &sac, NULL);
	sigaction(SIGTERM, &sac, NULL);
	sigaction(SIGINT,  &sac, NULL);
	sigaction(SIGQUIT, &sac, NULL);
	sigaction(SIGPIPE, &sac, NULL);

	printf("%s starting (duration <= %d seconds).\n", message, delay);
	nr = 0;
	do {
		cargv = argv + optind;
		while (optind < argc && strcmp(argv[optind], "--"))
			optind++;
		if (optind < argc)
			argv[optind++] = NULL;
		if ((c = fork()) < 0) {
			perror("fork");
			exit(1);
		}
		if (c == 0) {
			sac.sa_handler = catch;
			sigaction(SIGUSR1, &sac, NULL);
			pause();
			execvp(cargv[0], cargv);
			perror(cargv[0]);
			exit(1);
		}
		child[nr++] = c;
	} while (optind < argc);

	sleep(2);
	for (i = 0; i < nr; i++)
		kill(child[i], SIGUSR1);

	while (delay && childdied < nr) {
		delay = sleep(delay);
		if (terminate) {
			printf("%s terminated.\n", message);
			for (i = 0; i < nr; i++)
				kill(child[i], terminate);
			exit(1);
		}

		/* FIXME: collect and report child exit status */
	}

	for (i = 0; i < nr; i++)
		kill(child[i], signo);

	/* Wait up to 5 seconds for children to finish */
	delay = 5;
	while (delay && childdied < nr)
		delay = sleep(delay);

	if (childdied < nr) {
		printf("Children still running -- using brute force.\n");
		for (i = 0; i < nr; i++)
			kill(child[i], SIGKILL);
	}

	for (i = 0; i < nr; i++) {
		if ((c = wait(&status)) < 0) {
			perror("wait");
			exit(1);
		}

		if (WIFEXITED(status)) {
			const char	*blurb;

			if (mayexit) {
				if (WEXITSTATUS(status) == 0)
					continue;
				blurb = "with non-zero";
			} else {
				blurb = "prior to signal;";
			}
			printf("Child %d terminated %s exit status %d\n",
				c, blurb, WEXITSTATUS(status));
			goto fail;
		} else
		if (WIFSIGNALED(status) && WTERMSIG(status) != signo) {
			printf("Child %d died from signal %d (sent %d)\n",
				c, WTERMSIG(status), signo);
			goto fail;
		}
	}

	printf("%s passed.\n", message);
	return 0;

fail:
	printf("%s failed.\n", message);
	return 1;
}

static int
getsig(const char *spec)
{
	if (isdigit(spec[0]))
		return atoi(spec);
	if (!strcasecmp(spec, "INT"))
		return SIGINT;
	if (!strcasecmp(spec, "TERM"))
		return SIGTERM;
	if (!strcasecmp(spec, "KILL"))
		return SIGKILL;
	if (!strcasecmp(spec, "HUP"))
		return SIGHUP;
	if (!strcasecmp(spec, "ALRM"))
		return SIGALRM;

	printf("unexpected signal name %s\n", spec);
	exit(1);
}
